#!/usr/bin/perl

# $Id: edittarget.cgi 94 2009-02-19 16:29:16Z emmanuel $ 

use strict;
use warnings;

use Data::Dumper;

our %text;
our %config;
our %in;

require './iscsi-lib.pl';

# parse configuration files
my $conf=&parse_ietd_conf($config{'ietdconf'});


# ietd keywords
my %targetdefaults = (	
			MaxConnections			=> 1,
			InitialR2T			=> 'Yes',
			ImmediateData			=> 'No',
			MaxRecvDataSegmentLength	=> 8192,
			MaxXmitDataSegmentLength	=> 8192,
			MaxBurstLength			=> 262144,
			FirstBurstLength		=> 65536,
			DefaultTime2Wait		=> 2,
			DefaultTime2Retain		=> 20,
			MaxOutstandingR2T		=> 8,
			DataPDUInOrder			=> 'Yes',
			DataSequenceInOrder		=> 'Yes',
			ErrorRecoveryLevel		=> 0,
			HeaderDigest			=> 'CRC32C,None',
			DataDigest			=> 'CRC32C,None',
			Wthreads			=> 8,

) ;


##############################
# functions
sub addlun {
	# Display the form to add a new lun
	&header($text{'edittarget_addlun'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);

	# Display the lun details - used for add and edit.
	_print_lun('doaddlun');
}

sub doaddlun {
	# Apply the results from the add a new lun form
	&header($text{'edittarget_addlun'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);
	
	# define new lun number
	my ( $max ) = sort { $b <=> $a } keys %{$conf->{$in{'target'}}{'Luns'}};
	$max = -1 unless $max ne '';
	$max++;
	
	# cancel if invalid device
	if ( $in{'luntype'} ne 'nullio' and not -f $in{'lundev'} and not -b $in{'lundev'}) {
		fail("invalid file :'$in{'lundev'}'");
	}

	if (not $in{'IOMode'}) {
		$in{'IOMode'}='wthru'
	}

	# Set the field values 
	$conf->{$in{'target'}}{'Luns'}{$max}{'Path'} = $in{'lundev'};
	$conf->{$in{'target'}}{'Luns'}{$max}{'Type'} = $in{'luntype'};
        $conf->{$in{'target'}}{'Luns'}{$max}{'IOMode'} = $in{'IOMode'};
        $conf->{$in{'target'}}{'Luns'}{$max}{'ScsiId'} = $in{'lunscsiid'};
        $conf->{$in{'target'}}{'Luns'}{$max}{'ScsiSN'} = $in{'lunscsisn'};
	
	# Attempt to save the data to the ietd configuration file and display the outcome.
	eval { write_ietd_conf($config{'ietdconf'}, $conf) };
	if ($@) {
		fail($@);
	} else {
		print '<h3>' . $text{'edittarget_done'} . '</h3>'		
	}
        print "<br><br><hr>\n<a href=index.cgi>$text{auth_index}</a><br><BR>\n";
}

sub dellun {
	#Display the form to delete an existing lun.
        my $nextaction = "dodellun";
        &header($text{'edittarget_dellun'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);

        print &ui_form_start("edittarget.cgi", "get");
        print &ui_table_start($text{'edittarget_dellun'}, undef, 2);

        print &ui_table_row($text{'edittarget_targetname'}, $in{'target'} );
        print &ui_table_row($text{'edittarget_lunid'}, $in{'lun'} );
        print &ui_table_row($text{'edittarget_delfile'}, &ui_checkbox("delfile",'selected', '', 'selected') );

	# Hidden fields to keep track of state.
        print &ui_hidden("action", "$nextaction");
        print &ui_hidden("target", $in{'target'});
        print &ui_hidden("lun", $in{'lun'});

        print &ui_table_end();
        print &ui_form_end([ [ "$nextaction", $text{'edittarget_apply'} ] ]);
}

sub dodellun {
	# Apply deletion
	&header($text{'edittarget_dellun'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);
	
	# print "<pre>", Dumper \%in, "</pre>";
	
	# Delete the lun file if requested to do so.
	if ( $in{'delfile'} and ( -f $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'Path'} )) {
		print "<h3>Deleting $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'Path'} </h3>";
		# delete the file if it's not a device
		unlink $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'Path'} 
			or print "<h3>$text{'edittarget_delfileerror'} : $!</h3>";
	}
	
	delete $conf->{$in{'target'}}{'Luns'}{$in{'lun'}};

        # Attempt to save the data to the ietd configuration file and display the outcome.	
	eval { write_ietd_conf($config{'ietdconf'}, $conf) };
	if ($@) {
		fail($@);
	} else {
		print '<h3>' . $text{'edittarget_done'} . '</h3>'		
	}
        print "<br><br><hr>\n<a href=index.cgi>$text{auth_index}</a><br><BR>\n";
}


sub editlun {
	# Display the 'Edit Lun' form.
	&header($text{'edittarget_editlun'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),undef,undef,$text{'edittarget_lunid'}." $in{'lun'}");

        # Display the lun details - used for add and edit.
	_print_lun('doeditlun');
}

sub doeditlun {
	# apply edits
	&header($text{'edittarget_editlun'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);
	
	$conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'Path'} = $in{'lundev'};
	$conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'Type'} = $in{'luntype'};
	$conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'IOMode'} = $in{'IOMode'};
        $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'ScsiId'} = $in{'lunscsiid'};
        $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'ScsiSN'} = $in{'lunscsisn'};
	
        # Attempt to save the data to the ietd configuration file and display the outcome.
	eval { write_ietd_conf($config{'ietdconf'}, $conf) };
	if ($@) {
		fail($@);
	} else {
		print '<h3>' . $text{'edittarget_done'} . '</h3>'		
	}
        print "<br><br><hr>\n<a href=index.cgi>$text{auth_index}</a><br><BR>\n";
}

sub addtarget {
	# Display the 'Add Target' form
	&header($text{'edittarget_addtarget'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);

        # Display the target details - used for add and edit.
	_print_target('doaddtarget');
}

sub doaddtarget {
	# Apply the results returned from the 'Add Target' form
	&header($text{'edittarget_addtarget'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);

	# Validate the target name before adding.	
	if ( $in{targetname} eq "" ) {	
		print '<h3>' . $text{'edittarget_missingtargetname'} . '</h3>' 
	} elsif ( exists $conf->{$in{targetname}} ) {
		print '<h3>' .$in{targetname} .' '. $text{'edittarget_addexist'} . '</h3>' 
	} elsif  ( $in{'targetname'} =~ /(\w|\.|\+|-)+/ ) {
		$conf->{$in{targetname}} = {};
		if ( defined $in{targetalias} ) {
			$conf->{$in{targetname}}{Alias} = $in{targetalias} ;
		}

                # Attempt to save the data to the ietd configuration file and display the outcome.
		eval { write_ietd_conf($config{'ietdconf'}, $conf) };
		if ($@) {
			fail($@);
		} else {
			print '<h3>' . $text{'edittarget_done'} . '</h3>'		
		}
	} else {
		print  '<h3>' . $text{'edittarget_invalidtgtname'} . '</h3>'  
	}
	
        print "<br><br><hr>\n<a href=index.cgi>$text{auth_index}</a><br><BR>\n";
}


sub deltarget {
	# Display the 'Delete Target' form
	my $nextaction = "dodeltarget";
	&header($text{'edittarget_deltarget'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);

	# Check that there are no luns defined against the target before delete.
        if ( defined $conf->{$in{target}}{'Luns'} ) {
                print '<br><hr><br><h3>' . $text{'edittarget_notempty'} . '</h3>';
                print "\n<br><a href=index.cgi>$text{auth_index}</a><br><BR>\n";
        } else {

                print &ui_form_start("edittarget.cgi", "get");
                print &ui_table_start($text{'edittarget_deltarget'}, undef, 2);

                print &ui_table_row($text{'edittarget_targetname'}, $in{'target'} );

                print &ui_hidden("action", "$nextaction");
                print &ui_hidden("target", $in{'target'});
                print &ui_hidden("lun", $in{'lun'});

                print &ui_table_end();
                print &ui_form_end([ [ "$nextaction", $text{'edittarget_apply'} ] ]);

               print "<br><br><hr>\n<a href=index.cgi>$text{auth_index}</a><br><BR>\n";
	}
}

sub dodeltarget {
	# Apply deletion
	&header($text{'edittarget_deltarget'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);
	
	# Remove the target from in-memory cache
	delete $conf->{$in{'target'}};
	
        # Attempt to save the data to the ietd configuration file and display the outcome.
	eval { write_ietd_conf($config{'ietdconf'}, $conf) };
	if ($@) {
		fail($@);
	} else {
		print '<h3>' . $text{'edittarget_done'} . '</h3>'		
	}
        print "<br><hr><br><br>\n<a href=index.cgi>$text{auth_index}</a><br><br>\n";
}

sub edittarget {
	# Display the 'Edit Target' form
	&header($text{'edittarget_title'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);

        # Display the target details - used for add and edit.
	_print_target('doedittarget');	
}

sub doedittarget {
	# Apply edits
	&header($text{'edittarget_title'}, "", undef, 1, 1, 0, &help_search_link("ietd", "man", "doc"),);
	
	# print "<pre>",Dumper $conf, "</pre>";
	
	
	$conf->{$in{'target'}}{'Alias'} = $in{'targetalias'}
		if  $in{'targetalias'} =~ /(\w|\.|\+|-)+/ ;
	
	if ( $in{'targetname'} =~ /(\w|\.|\+|-)+/ 
		and $in{'target'} ne $in{'targetname'} ) {
		$conf->{$in{'targetname'}} = $conf->{$in{'target'}} ;
		delete $conf->{$in{'target'}};
	}
	
	# print "<pre>",Dumper $conf, "</pre>";
	
        # Attempt to save the data to the ietd configuration file and display the outcome.
	eval { write_ietd_conf($config{'ietdconf'}, $conf) };
	if ($@) {
		fail($@)
	} else {
		print '<h3>' . $text{'edittarget_done'} . '</h3>'		
	}
    print "<br><br><hr>\n<a href=index.cgi>$text{auth_index}</a><br><BR>\n";
}


########################
sub _print_lun {

	my $nextaction=shift;

	# get LUN info from configuration OR CGI params
	my $lunfile = $in{'lundev'} || $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'Path'};
	my $luntype = $in{'luntype'} || $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'Type'};
	my $luniomode = $in{'IOMode'} || $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'IOMode'};
	my $lunscsiid = $in{'lunscsiid'} || $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'ScsiId'};
    my $lunscsisn = $in{'lunscsisn'} || $conf->{$in{'target'}}{'Luns'}{$in{'lun'}}{'ScsiSN'};
	
	my $createlun = "<a href=\"createlun.cgi?target=$in{'target'}\">$text{'edittarget_createdev'}</a>";
	
	if ( $in{action} eq 'editlun' ) {
	$createlun = "<a href=\"createlun.cgi\?action=resizedev&target=$in{'target'}&lundev=$lunfile&lun=$in{lun}\">$text{'edittarget_resizedev'}</a>"
	}
	
	print qq(<script src="tools.js"></script>);

	if ( $luntype !~ /fileio/ ) {
		print qq(
        <script language="JavaScript" type="text/javascript">
        window.onload=function () {
        	var elements = document.getElementsByName('luntype');
        	elements[0].onchange = setLunParams;
        	setLunParams();
        };
        </script>);
	}
	
	print &ui_form_start("edittarget.cgi", "get");
	print &ui_table_start($text{'lun_header'}, undef, 2);

	print &ui_table_row($text{'edittarget_luntype'},
	        &ui_select("luntype", $luntype, 
			[ ["nullio", $text{'edittarget_luntypenull'} ],
			  ["blockio", $text{'edittarget_luntypeblock'} ],
			  ["fileio", $text{'edittarget_luntypefile'} ] ]));

        print &ui_table_start("", "id='lunparams' width='100%'", 2);

	print &ui_table_row($text{'edittarget_lundev'}, &ui_textbox("lundev", $lunfile, 40)."&nbsp;".&file_chooser_button("lundev") . $createlun );
	
		
	print &ui_table_row($text{'edittarget_lunscsiid'}, &ui_textbox("lunscsiid", $lunscsiid, 40));
	print &ui_table_row($text{'edittarget_lunscsisn'}, &ui_textbox("lunscsisn", $lunscsisn, 40));
	print &ui_table_row($text{'edittarget_iomode'},
        	&ui_radio("IOMode", $luniomode, 
			[ ["wb", $text{'edittarget_iomodewb'} ."<br>" ],
			  ["wthru", $text{'edittarget_iomodewt'}."<br>" ],
			  ["ro", $text{'edittarget_iomodero'} ."<br>" ] ]));

	print &ui_hidden("action", $nextaction);
	print &ui_hidden("target", $in{'target'});
	print &ui_hidden("lun", $in{'lun'});
        print &ui_table_end();

	print &ui_table_end();
	print &ui_form_end([ [ "$nextaction", $text{'edittarget_apply'} ] ]);

        print "<hr>\n<a href=index.cgi>$text{auth_index}</a><br><br>\n";
}


sub _print_target {
	my $nextaction=shift;

        my $host = "";
        my @hostarray = split(/\./, `hostname -f`);
		chomp @hostarray;

        for (my $i = @hostarray; $i > 0; $i--) { $host = $host.'.'.$hostarray[$i-1]; }

	my $targetname=$in{'target'} || "iqn.2001-04:$host.TARGETNAME" ;
	my @temp =  split ( /.*:/, $targetname );
	my $targetalias=$conf->{$in{'target'}}{'Alias'}|| $temp[$#temp];

        print &ui_form_start("edittarget.cgi", "get");
        print &ui_table_start($text{'edittarget_targetinfo'}, undef, 2);

        print &ui_table_row($text{'edittarget_targetname'}, &ui_textbox("targetname", $targetname, 50));
        print &ui_table_row($text{'edittarget_targetalias'}, &ui_textbox("targetalias", $targetalias, 50));

        print &ui_hidden("action", $nextaction);
        print &ui_hidden("target", $in{'target'});

        print &ui_table_end();
        print &ui_form_end([ [ "$nextaction", $text{'edittarget_apply'} ] ]);

        print "<hr>\n<a href=index.cgi>$text{auth_index}</a><br><br>\n";
}

##############################
# failure!

sub fail {
	my $message=shift;
	print '<h3>' . $text{'edittarget_failed'} . $message . '</h3>';
	print "<hr>\n<a href=index.cgi>$text{auth_index}</a><br><br>\n";
	# footer
	&footer("/", $text{'index'});
	exit;
}

##############################

# actions
my %actions = (
	addlun 		=> \&addlun,
	doaddlun 	=> \&doaddlun,
	dellun 		=> \&dellun,
	dodellun 	=> \&dodellun,
	editlun		=> \&editlun,
	doeditlun	=> \&doeditlun,
	addtarget 	=> \&addtarget,
	doaddtarget	=> \&doaddtarget,
	deltarget 	=> \&deltarget,
	dodeltarget => \&dodeltarget,
	edittarget	=> \&edittarget,
	doedittarget	=> \&doedittarget,
) ;

##############################

if ( exists $actions{$in{action}} ) {
	$actions{$in{action}}->();
} else {
	edittarget();
}

# test
# print "<pre>";
# print Dumper \%in;
# print "</pre>";

# footer
&footer("/", $text{'index'});


# end
exit;
